#ifndef _FIFO_HH
#define _FIFO_HH


#include <iostream>


using namespace std;

#ifdef FIFO_USING_LIST

#include <list>
#include <iterator>

template <class T> class FIFO;

template <class T>
class FIFOL
{
protected:
  list<T> data;

public:
  FIFOL() {}
  ~FIFOL() {}

  void push( const T & elm ) 
  { 
    data.push_back( elm ); 
  }
  T pop() 
  { 
    T elm(data.front());
    data.pop_front();
    return elm;
  }
  bool empty() const
  {
    return data.empty();
  }
  size_t size() const
  {
    return data.size();
  }
  
  ostream & affiche( ostream &out) const
  {
    out << "[ ";
    if ( !empty() )
      {
	typename list<T>::const_iterator it(data.end());
	--it;
	if ( it!=data.begin() )
	  copy( data.begin(), it, ostream_iterator<T>( out, ", " ) );
	out << *it;
      }
   out << " ]";
    return out;
  }
  
  bool compare( const FIFO<T> &fifo ) const
  {
    list<int>::const_iterator it( data.begin() );
    bool init( true );
    while ( it!= data.end() )
      {
	if ( *(it++) != fifo.next(init) )
	  return false;
	init = false;
      }
    return true;
  }
	
};

#endif

#include <vector>

template <class T>
class FIFO
{
protected:
  int n_base;
  int nb;
  T * begin, * last;
  T * data, *end;

  bool is_full() const
  {
    return last == begin;
  }

  void resize()
  {
    T *ptr;
    int pos( begin - data );
    cerr << "realloc ... " << flush;
    if ( (ptr = (T *)realloc( data, (nb<<1)*sizeof(T) )) == 0 )
      {
	free(data);
	throw "FIFO::push() : Erreur sur realloc()";
      }
    data = ptr;
    cerr << "[OK]" << endl;
    // deplacer (nb - pos) elements en bout de tableau : data + 2*nb - (nb - pos)
    if ( pos != nb )
      {
	last = data + pos;
	begin = last + nb;
	cerr << "memmove ... " << flush;
	memmove(begin, last, (nb - pos)*sizeof(T));
	cerr << "[OK]" << endl;
      }
    else
      {
	begin = data + (nb<<1);
	last = data + nb;
      }
    nb <<= 1;
    end = data + nb;
  }



public:
  FIFO( int n_b=100 ) : n_base(n_b), nb(n_base)
  {
    data =(T *)malloc( nb*sizeof(T) );
    begin = last = end = data + nb;
    if ( data==0 )
      throw "FIFO::FIFO() : Erreur d'allocation memoire";
  }
  ~FIFO() 
  {
    free(data);
  }

  void add( const T & elmt )
  {
    if ( last >= end )
      last = data;
    *(last++) = elmt;
    if ( is_full() )
      {
// 	affiche( cerr, true );
// 	cerr << endl;
	resize();
// 	affiche( cerr );
// 	cerr << endl;
      }
  }
  T first()
  {
    if ( begin >= end )
      begin = data;
    return *(begin++);
  }
  bool empty() const
  {
    return begin == last;
  }

  size_t size() const
  {
    return begin<=last ? last - begin : nb + last - begin; // data + nb - begin + last - data
  }

  ostream & affiche( ostream & out, bool force=false ) const
  {
    T *ptr = ( begin == end ? data : begin );
    out << "[ ";
    if ( !empty() || force)
      {
	out << *(ptr++);
	while ( ptr!=last )
	  {
	    if ( ptr==end )
	      ptr = data;
	    out << ", " << *(ptr++);
	  }
      }
    out << " ]";
    return out;
  }

  int next( bool init = false ) const
  {
    static T *ptr;

    if ( init ) 
      ptr = begin;
    if ( ptr==end )
      ptr = data;
    return *(ptr++);
  }
};
  
  

#endif
  
template<class T>
ostream & operator << ( ostream & out, const FIFO<T> & fifo )
{
  return fifo.affiche( out );
}




