#ifndef __LIST_HH__
#define __LIST_HH__

#include "tab.hh"


template <class X>
class list : private tab1d<X>
{
private:
  // The number of elements in the list
  int cardinal;
public:
  /// Constructor
  list():cardinal(0)
  {}
  /// Function to clear all the list
  void reinit();
  /// Return the number of elements of the list
  int give_cardinal() const;
  /// Return true if the list is empty
  bool empty() const;
  
  typedef enum {first_X,last_X,end_X} position;
  class iterator
  {
  private:
    int current;
    list<X> * obj;
    
  public:
    iterator():current(0),obj(0)
    {}
    iterator(list<X>& l,position pos=list<X>::first_X);
    iterator &operator ++();
    iterator &operator ++(int); 
    
    X &operator *();

    bool operator<(const iterator&) const;
    bool operator>(const iterator&) const;
    bool operator<=(const iterator&) const;
    bool operator>=(const iterator&)const;
    bool operator==(const iterator&) const;
    bool operator!=(const iterator&) const;
  };
  
  /// Give an iterator positioned on the first element of the list
  iterator & first();
  /// Give an iterator positioned on the last element of the list
  iterator & last();
  /// Give an iterator positioned on the element of the list after the last one
  iterator & end();
  /// Add an element to the list
  void insert(const X&);
  /// Check if an element is on the list
  bool is_in(const X&);
  /// Remove an element from the list
  bool remove(const X&);
  /// Remove from the list the element pointed by the iterator
  void remove(iterator&);
};
  
#endif // __LIST_HH__
