/** \file color_and_border_set.hh
    \brief File to manage color and border data of the pyramid */

#ifndef __COLOR_AND_BORDER_SET_HH__
#define __COLOR_AND_BORDER_SET_HH__

#include "value_set.hh"
#include "vertex_data_color.hh"
#include "border_data_deriche.hh"

/** Class to manage color and border data of the pyramid */
class color_and_border_set: public value_set
{
private:
  // Pointor on the vertex dataset
  vertex_data_color *v_data;
  // Pointor on the border dataset
  border_data_deriche *b_data;

public:
  // Constructor
  color_and_border_set(vertex_data_color &v, border_data_deriche &b):v_data(&v),b_data(&b)
  {}

  /// Function to initialize the data_set during the construction of the pyramid
  void init(combi_map_base &);
  /// Function to manage the data when an edge will be contracted
  void contract_edge(int, dart, dart, vertex, vertex);
  /// Function to manage the data when an empty self-loop will be remove
  void remove_empty_self_loops(dart, dart, vertex);
  /// Function to manage the data when a double edge will be remove
  void remove_double_edges(int, dart, dart);
  /// Function to manage the data when vertex set is updated
  void update_vertex_set(int, vertex, vertex);
};

#endif // __COLOR_AND_BORDER_SET_HH__
