/** \file combi_map_base.hh
    \brief File for the combinatorial maps of the base of the pyramid */

#ifndef __COMBI_MAP_BASE_HH__
#define __COMBI_MAP_BASE_HH__

#include "combi_map_priv.hh"
#include "point.hh"

class combi_map_priv;

/** Model class for the combinatorial maps of the base of the pyramid */
class combi_map_base: public combi_map
{
protected:
    // The width and height dimention
  int dimx,dimy;

public:

  combi_map_base(int dim_x,int dim_y):dimx(dim_x),dimy(dim_y)
  {}
  /// A pure virtual function to access to the sigma of one dart of the combinatorial map
  virtual dart sigma(dart) const=0;

  /// optimization of the phi function of the mother class.
  // to modify if you wish to not use the implicit relationship alpha(b)=-b  
  dart phi(dart b)
  {
    return sigma(-b);
  }
  virtual vertex mu(dart d) const
  {
    point P;
    give_pixel(d,P);
    return P.y*dimx+P.x+1;
  }
  /// A pure virtual function to access to the alpha of one dart of the combinatorial map
  // to modify if you wish to not use the implicit relationship alpha(b)=-b
  virtual dart alpha(dart b) const
  {
    return -b;
  }
  /// Function that compute the variation of orientation between two darts
  virtual int compute_index_orientation(dart,dart)=0;  
  /// A pure virtual function that give the coordinates of the pixel defined by the dart  
  virtual void give_pixel(dart,point &p)const =0 ;
  /// A pure virtual function that give the coordinates of the vertex of the map defined by the dart
  virtual void give_pointel(dart,point &p) const =0;
  /// A pure virtual function that give the orientation of the dart
  virtual move_type give_move(dart) const =0;
  /// A pure virtual function that initialize a combinatorial_map from a combi_map_base
  virtual void init_combi_map_priv(combi_map_priv &c_map) const =0;
  /// A pure virtual function that give the width of the base map
  virtual int give_width()const =0 ;
  /// A pure virtual function that give the height of the base map
  virtual int give_height()const =0 ;
};



#endif // __COMBI_MAP_BASE_HH__
