#include "combi_map_base_exp.hh"


combi_map_base_exp::combi_map_base_exp(int _dimx,int _dimy):combi_map_base(_dimx,_dimy)
{
  dart b,up,left,right,down;
  unsigned long Background;
  

  nb_edge = dimx*(dimy+1)+(dimx+1)*dimy;
  Background=dimx*dimy+1;

  up=1;
  left= dimx*(dimy+1)+1;
  right=left+1;
  down=-(dimx+1);
  for(int j=0;j<dimy;j++)
  {
     /*  left column */
    sigma_base[up]=-left;
    sigma_base[-left]=down;
    sigma_base[down]=right;
    sigma_base[right]=up;

    for(int i=0;i<dimx-1;i++)
    {
      up++;down--;right++;
      sigma_base[up]=-right+1;
      sigma_base[-right+1]=down;
      sigma_base[down]=right;
      sigma_base[right]=up;
    }
    up++;down--;
    left=right+1;
    right+=2;
  }

  /* Encoding of the background vertex */

  left=dimx*(dimy+1)+1;
  sigma_base[left]=-1;
  for(b=1;b<dimx;b++)
    sigma_base[-b]=-b-1;

  right=left+dimx;
  sigma_base[-dimx]=-right;
  for(b=right;b<=nb_edge-dimx-1;b+=dimx+1)
    sigma_base[-b]=-b-dimx-1;

  right=  dimx*(dimy+1);
  sigma_base[-nb_edge]=right;
  for(b=right;b>right-dimx+1;b--)
    sigma_base[b]=b-1;

  left=right-dimx+1;
  sigma_base[left]=nb_edge-dimx;
  for(b=nb_edge-dimx;b>dimx*(dimy+1)+1;b-=(dimx+1))
    sigma_base[b]=b-dimx-1;
}

dart combi_map_base_exp::sigma(dart d) const
{
  return sigma_base(d);
}

int combi_map_base_exp::give_nb_edge() const
{
  return nb_edge;
}
unsigned long combi_map_base_exp::give_background() const
{
  return dimx*dimy+1;
}

int combi_map_base_exp::compute_index_orientation(dart d1,dart d2)
{
  int nb = dimx*(dimy+1);
  if (d1>0)
  {
// d1 is left
    if (d1<=nb)
    {
      if (d2>0)
      {
        if (d2<=nb)
	  return 0;
        return 1;
      }
      if (-d2<=nb)
        return 0;
      return -1;
    }
    
// d1 is up
    if (d2>0)
    {
      if (d2<=nb)
        return -1;
      return 0;
    }
    if (-d2>nb)
      return 0;
    return 1;
  }

// d1 is right
  if (-d1<=nb)
  {
    if (d2>0)
    {
      if (d2<=nb)
        return 0;
      return -1;
    }
    if (-d2<=nb)
      return 0;
    return 1;
  }
    
// d1 is down
  if (d2>0)
  {
    if (d2<=nb)
      return 1;
    return 0;
  }
  if (-d2>nb)
    return 0;
  return -1;
}

void combi_map_base_exp::give_pixel(dart d,point &p) const
{
  int horizontal=dimx*dimy, val;
  if (d > 0)
  {
    if(d <= horizontal+dimx)
    {
      if (d > horizontal)
        throw point_error("combi_map_base_imp give_pixel(dart,point&) point outside of the picture");
      p.x=(d-1)%dimx;
      p.y=(d-1)/dimx;
      return;
    }
    val = (d-horizontal-dimx);
    if (val%(dimx+1)==1)
      throw point_error("combi_map_base_imp give_pixel(dart,point&) point outside of the picture");
    p.x=(val-2)%(dimx+1);
    p.y=(val-2)/(dimx+1);
    return;
  }
  if(-d <= horizontal+dimx)
  {
    if (-d <= dimx)
      throw point_error("combi_map_base_imp give_pixel(dart,point&) point outside of the picture");
    p.x=(-d-dimx-1)%dimx;
    p.y=(-d-dimx-1)/dimx;
    return;
  }
  val = (-d-horizontal-dimx);
  if (val%(dimx+1)==0)
    throw point_error("combi_map_base_imp give_pixel(dart,point&) point outside of the picture");
  p.x=(val-1)%(dimx+1);
  p.y=(val-1)/(dimx+1);
  return;

}
void combi_map_base_exp::give_pointel(dart d,point &p) const
{
  int nb = dimx*(dimy+1), val;
  if (d<0)
  {
    if ( -d<=nb )
    {
      val = -d-1;
      p.x=val%dimx;
      p.y=val/dimx;
      return;
    }
   val = -d-nb-1;
   p.x=val%(dimx+1);
   p.y=val/(dimx+1);
   return;
  }
  if (d<=nb)
  {
    p.x=(d-1)%dimx+1;
    p.y=(d-1)/dimx;
    return;
  }
  val = d-nb+dimx;
  p.x=val%(dimx+1);
  p.y=val/(dimx+1);
  return;
}
move_type combi_map_base_exp::give_move(dart d) const
{
  int horizontal=dimx*(dimy+1);
  if (d > 0)
  {
    if(d <= horizontal)
      return left_move;
    return up_move;
  }
  if(-d <= horizontal)
    return right_move;
  return down_move;
}

void combi_map_base_exp::init_combi_map_priv(combi_map_priv &c_map) const
{
  int b;
  c_map.nb_edge = nb_edge;
  c_map.background = dimx*dimy+1;
  for(b=-nb_edge;b<0;b++)
  {
    c_map.sig[b]=sigma_base(b);
    c_map.alph[b]=-b;
  }
  for(b=1;b<=nb_edge;b++)
  {
    c_map.sig[b]=sigma_base(b);
    c_map.alph[b]=-b;
  }
  if (c_map.vertex_set.empty())
    c_map.vertex_set.reinit();
  for(int i=1;(unsigned int)i<=c_map.background;i++)
  {
    c_map.m[i] = i;
    c_map.m_1[i] = i;
    c_map.vertex_set.insert(i);

    b=sigma_base(i);
    while (i!=b)
    {
      c_map.m[b] = i;
      b = sigma_base(b);
    }
  }
}

int combi_map_base_exp::give_width()const
{
  return dimx;
}
int combi_map_base_exp::give_height()const
{
  return dimy;
}
