/** \file combi_map_base_exp.hh
    \brief File that manage the combinatorial maps of the base of the pyramid by storing sigma permutation */

#ifndef __COMBI_MAP_BASE_EXP_HH__
#define __COMBI_MAP_BASE_EXP_HH__

#include "combi_map_base.hh"

/** Class that manage the combinatorial maps of the base of the pyramid by
storing sigma permutation */
class combi_map_base_exp: public combi_map_base
{
private:
  // Tab of dart that hold the sigma of the map
  tab_dart sigma_base;
  // number of edge of the map
  int nb_edge;

public:
  /// Constructor
  combi_map_base_exp(int dimx,int dimy);
  /// Virtual destructor
  virtual ~combi_map_base_exp()
  {}
  /// Function to access to the sigma of one dart of the combinatorial map
  dart sigma(dart) const;
  /// Function that compute sigma precedent of the alpha of the dart
  dart phi(dart b);
  /// Function that give the number of edge
  int give_nb_edge() const;
  /// Function that give the backround value
  unsigned long give_background() const;

  /// Function that compute the variation of orientation between two darts
  int compute_index_orientation(dart,dart);
  /// Function that give the coordinates of the pixel defined by the dart
  void give_pixel(dart,point &p) const;
  /// Function that give the coordinates of the vertex of the map defined by the dart
  void give_pointel(dart,point &p)  const;
  /// Function that give the orientation of the dart
  move_type give_move(dart) const ;
  /// Function that initialize a combinatorial_map from a combi_map_base_exp
  void init_combi_map_priv(combi_map_priv &c_map) const;
  /// Function that give the width of the base map
  int give_width()const;
  /// Function that give the height of the base map
  int give_height()const;

};

#endif // __COMBI_MAP_BASE_EXP_HH__
