#include "dynamic_kernel.hh"

dynamic_kernel::dynamic_kernel(water_value& values,image& img,image& grad,int max):data(&values),dynamic(0),debug(1,img.Width(),img.Height()),gradient(grad),max_level(max),level(7)
{
  image::iterator it_tmp(img);
  
  for(image::iterator ite(debug);
      ite<debug.end();
      ite++,it_tmp++)
    {
      if(img.Dim()==1)
	{
	  *ite=255;
	}
      //	for(int i=0;i<3;i++)

      else
	for(int i=0;i<3;i++)
	  ite[i]=it_tmp[i];      
    }
}

void dynamic_kernel::debug_output_image(combi_map_priv map,water_value values)
{
  unsigned char C[3];
  image res(debug);
  int nb_level=map.pyr->give_max_level();
  segment_iterator it_seg;
  image::iterator it_im(res);
  point p;
  int value,dynamic;
  dart b,m_b;
  vertex mu,mu_1,background=map.give_background();
  char name[256];

  image::iterator grad_ite(gradient);
  int min_debug;
  
  cerr << "level : "<<level<<endl;
  sprintf(name,"dynamic%d.ppm",level);
  level++;
  
  for(dart_iterator ite=map.pyr->first_dart(nb_level);
      ite<map.pyr->end_dart(nb_level);
      ite++)
    {
      b=*ite;
      mu=map.mu(b);
      if (mu==background)
        continue;
      m_b = map.alpha(b);
      mu_1 = map.mu(m_b);
      if (mu==mu_1)
	continue;
      //      if(!contracted(b) && !contracted(m_b))
      //	continue;
      if(b<m_b && mu_1 != background)
	continue;
      combinatorial_map::segment seg(b,m_b,nb_level,map.pyr);
      dynamic=values.dynamic(b);
      value=values.pass_value(b);
      min_debug=values.min(b);
      if(dynamic==-1)
	{
	  dynamic=values.dynamic(map.alpha(b));
	  value=values.pass_value(map.alpha(b));
	  min_debug=values.min(b);
	}
      C[0]=dynamic;
      C[1]=value;
      it_seg = seg.first();
      do
      {
        map.pyr->give_pixel(*it_seg,p);
	it_seg++;
        grad_ite.move_to(p.y, p.x);
	it_im.move_to(p.y, p.x);
	*it_im=255-dynamic;
	C[2]=*grad_ite;

        /*for(int i=0;i<3;i++)
	  it_im[i] = C[i];*/
      }
       while (it_seg!=seg.end());
    }
  res.save(name);
}
void dynamic_kernel::compute_kernel(combi_map_priv & map)
{
  vertex v,vp,f_v,f_vp,background(map.give_background());
  dart b,mb;
  int dyn;


  data->compute_dynamics(dynamic);
  cerr<<"Updating dynamics : "<<dynamic<<endl;


  nb_contracted=0;
  contracted.reinit();
  // Each vertex is initially its own father
  set_father(map);

  int max_level=map.pyr->give_max_level();
  dart_iterator end_dart(map.pyr->end_dart(max_level));
  
  for(dart_iterator ite=map.pyr->first_dart(max_level);
      ite<end_dart;
      ite++)
    {
      b=*ite;
      dyn=data->dynamic(b);
      if(dyn == -1)
	continue;
      if(dyn !=dynamic)
	continue;
      mb=map.alpha(b);

      if(contracted(mb))
	continue;

      v=map.m(b);
      vp=map.m(mb);

      if(v==vp|| v==background || vp==background)
	continue;


      f_v=find_compress(v);
      f_vp=find_compress(vp);
      if(f_v == f_vp)
	continue;

      Father[f_v]=f_vp;
      contract_edge(b,mb);
      cerr <<"("<<b<<","<<mb<<")"<<endl;
    }
  if(nb_contracted)
    {
      cerr << "Dynamic : "<<dynamic<<", nb contracted : "<<nb_contracted;
      debug_output_image(map,*data);
    }
}
bool dynamic_kernel::continue_computation()
{
  return (dynamic<254)&&(level<max_level);
}
