/** \file equivalent_cleaning.hh
    \brief File used to determine and remove all the small vertices */

#ifndef __EQUIVALENT_CLEANING_HH__
#define __EQUIVALENT_CLEANING_HH__

#include "equivalent_mont_jolion.hh"
#include "border_data_deriche.hh"
#include "vertex_data_color.hh"

/** Model class used to determine and remove all the small vertices */
class equivalent_cleaning: public equivalent_mont_jolion
{
private:
  // minimum size of the area
  double size_threshold;
public:
  /// Constructor
  equivalent_cleaning(const vertex_data_color &v, const border_data_deriche &b):equivalent_mont_jolion(v, b),size_threshold(10)
  {}
  /// Function to know if the two vertices must be merged
  bool lambda(int,vertex,vertex,dart,dart);
  /// Function to compute the surviving criterion of a vertex
  /// That will be minimized during the reduction process
  double value(vertex);
  /// Function to compute the merging interset of between two vertices
  /// That will be minimized during the reduction process
  double interest(int,vertex,vertex,dart,dart);


  double give_size_threshold()
  {
    return size_threshold;
  }
  double set_size_threshold(const double &d)
  {
    return size_threshold=d;
  }
};

#endif // __EQUIVALENT_PRESERVE_EDGE_HH__
