#include "export.hh"
#include <list_label.hh>


#define RIGHT_NEIGHBOUR 1
#define LEFT_NEIGHBOUR  2
#define UP_NEIGHBOUR    4
#define NODE_CONST      8


void draw_descartes_edge(point &p_prev,point &p,image::iterator&it_im);

extern void export_to_descartes(combinatorial_map &c_map,char *file_name,draw_type type)
{

  image res(1,c_map.give_width()+1,c_map.give_height()+1);
  image::iterator it_im(res);
  segment seg;
  segment_iterator it_seg;
  dart_iterator it_dart = c_map.give_dart_first();
  point p, p_prev;
  dart b,m_b;
  
  do
  {
    b = *it_dart++;
    m_b = c_map.alpha(b);
    if (b<m_b)
      continue;
    if (type==draw_single && c_map.mu(b)==c_map.mu(m_b))
      continue;

    seg = c_map.give_segment(b);
    it_seg = seg.first();
    it_seg.give_point(p_prev);
    it_im.move_to(p_prev.y, p_prev.x);
    *it_im |= NODE_CONST;
    do
    {
      it_seg++;
      it_seg.give_point(p);
      draw_descartes_edge(p_prev,p,it_im);
      p_prev.x=p.x;
      p_prev.y=p.y;
    }
    while (it_seg!=seg.last());
    c_map.give_point(m_b,p);
    draw_descartes_edge(p_prev,p,it_im);
    *it_im |= NODE_CONST;
  }
  while (it_dart<c_map.give_dart_end());
  res.save(file_name);
}

void draw_descartes_edge(point &p_prev,point &p,image::iterator&it_im)
{
  int dx = p.x-p_prev.x, dy;

  if (dx<0)
  {
    *it_im |= LEFT_NEIGHBOUR;
    it_im.move_to(p.y, p.x);
    *it_im |= RIGHT_NEIGHBOUR;
    return;
  }
  if (dx>0)
  {
    *it_im |= RIGHT_NEIGHBOUR;
    it_im.move_to(p.y, p.x);
    *it_im |= LEFT_NEIGHBOUR;
    return;
  }
  dy = p.y-p_prev.y;
  if (dy<0)
  {
    *it_im |= UP_NEIGHBOUR;
    it_im.move_to(p.y, p.x);
    return;
  }
  it_im.move_to(p.y, p.x);
  *it_im |= UP_NEIGHBOUR;
}

void give_bounding_box(combinatorial_map &c_map,dart sig,dart end_sig,point &min,point &max);
void put_binmap_edge(point &p_prev,point &p,unsigned char *buf,const point &min,int w_buf);
void scan_bitmap_buffer(unsigned char*,int,const point&,int,int,int,int,image&);

void export_to_bitmap(combinatorial_map &c_map,vertex v,char *file_name)
{
  point min, max, p, p_prev;
  dart sig,end_sig;
  segment seg;
  segment_iterator it_seg;


  // find the bounding rectangle of the area
  sig=end_sig=c_map.give_neighbor_dart(v);
  give_bounding_box(c_map,sig,end_sig,min,max);

  // Store the inside-outside information of the border into a buffer to be readable by a scanline
  int dimx(max.x-min.x+1),dimy(max.y-min.y),size=(dimx*dimy+7)>>3,w_buf=dimx,dx,dy;
  unsigned char *buf,val;
  buf = new unsigned char [size];
  memset(buf, 0, size);
  image res(1,dimx-1,dimy);

  val=0;
  dx = dimx+dimx-1;
  while ((dx>>(++val)) != 0);
  dx = val;
  val=0;
  dy = dimy+dimy+1;
  while ((dy>>(++val)) != 0);
  dy = val;

  sig=c_map.alpha(sig);
  sig=c_map.give_alpha_base(sig);  
  c_map.give_point(sig,p_prev);
  sig=end_sig;
  do
  {
    sig = c_map.sigma(sig);
    seg = c_map.give_segment(sig);
    it_seg = seg.first();
    do
    {
      it_seg.give_point(p);
      put_binmap_edge(p_prev,p,buf,min,w_buf);
      it_seg++;
    }
    while (it_seg!=seg.end());
  }
  while (sig!=end_sig);

  // Scan the binary buffer to generate the inside cell
  scan_bitmap_buffer(buf,size,min,dimx,dx,dimy,dy,res);
  res.save(file_name);
  
  delete [] buf;
}
void give_bounding_box(combinatorial_map &c_map,dart sig,dart end_sig,point &min,point &max)
{
  point p;
  dart alpha_sig;
  segment seg;
  segment_iterator it_seg;

  c_map.give_point(sig,min);
  max.x=min.x;
  max.y=min.y;
  do
  {
    sig=c_map.sigma(sig);
    if (c_map.is_open_parenthesis(sig))
    {
      alpha_sig = c_map.alpha(sig);
      sig=c_map.sigma(sig);
      while(sig!=alpha_sig)
        sig=c_map.sigma(sig);
      sig=c_map.sigma(sig);
    }
    seg = c_map.give_segment(sig);
    it_seg = seg.first();
    do
    {
      it_seg.give_point(p);
      if (p.x < min.x)
        min.x = p.x;
      else
        if(p.x > max.x)
          max.x = p.x;      
      if (p.y < min.y)
        min.y = p.y;
      else
        if(p.y > max.y)
          max.y = p.y;      
      it_seg++;
    }
    while (it_seg!=seg.end());


  }
  while (sig!=end_sig);
}

void put_binmap_edge(point &p_prev,point &p,unsigned char *buf,const point &min,int w_buf)
{
  int dx = p.x-p_prev.x, dy, offset, shift, x, y;
  if (dx!=0)
  {
    p_prev.x=p.x;
    p_prev.y=p.y;
    return;
  }

  dy = p.y-p_prev.y;
  if (dy<0)
  {
    x = p_prev.x-min.x;
    y = p_prev.y-min.y-1;
    offset = y*w_buf+x;
    shift = offset&7;
    offset >>= 3;
    buf[offset] ^= (1<<shift);
    p_prev.x=p.x;
    p_prev.y=p.y;
    return;
  }
  x = p.x-min.x;
  y = p.y-min.y-1;
  offset = y*w_buf+x;
  shift = offset&7;
  offset >>= 3;
  buf[offset] ^= (1<<shift);
  p_prev.x=p.x;
  p_prev.y=p.y;
}

void scan_bitmap_buffer(unsigned char *buf, int size, const point &min, int dimx, int dx, int dimy, int dy,image &res)
{
  image::iterator it_im(res.begin());
  unsigned char *pt_buf=buf;
  bool open=false;
  int i=size, x=1;

  while(i--)
  {
    for(int val=1;val<255;val<<=1)
    {
      if(x++==dimx)
      {
         x=1;
         if (((*pt_buf)&val) != 0)
           open = !open;
	 continue;
      }
      if (((*pt_buf)&val) != 0)
        open = !open;
      if (open==false)
      {
        it_im++;
	continue;
      }
      *it_im++ = 255;
    }
    pt_buf++;
  }
}




// Compute the coordinates of the 1-cell in the bounding box referencial
// return true if the 1-cell is vertical and false if horizontal
bool give_1_cell_coordinate(dart,combinatorial_map&,int,point&,point&);
// Compute the coordinates of the 1-cell in the bounding box referencial
void give_2_cell_coordinate(dart,combinatorial_map&,int,point&,point&);
// Compute the alpha permutation for contracted dart at a given level
dart compute_alpha_contract(dart,int,combinatorial_map&);
// Compute the byte represent the cell in Lachaud format
int give_lachaud_cell(int,int,int,int);
// get the cell that represent the given dart and write it on the ofstream
// return the number of cell writed
int get_n_write_cell(dart,ofstream&,combinatorial_map&,int,int,int,point&,tab_sym_bool&);

void export_to_lachaud(combinatorial_map &c_map,vertex v,char *file_name)
{
  // find the bounding rectangle of the area
  ofstream out(file_name);
  dart b=c_map.give_neighbor_dart(v);
  point min,max;
  give_bounding_box(c_map,b,b,min,max);

  // Compute the shift value
  int i,dx,dy,dimx,dimy,val=0, pos,nb_cell=0;
  dimx = max.x-min.x;
  dx = ((dimx<<1)+1)>>1;
  while ((dx>>(++val)) != 0);
  dx = val;
  val=0;
  dimy = max.y-min.y;
  dy = ((dimy<<1)+1)>>1;
  while ((dy>>(++val)) != 0);
  dy = val;

  // Set the header of the lachaud file
  i = (dimx<<1)+1;
  out << i << ' ';
  i = (dimy<<1)+1;
  out << i << endl;
  pos = out.tellp();
  out << "                " << endl;
  out << 3 << ' ' << dx << ' ' << dy << ' ' << 0 << endl;

  // Find and compute the cells in the map and put it in the file
  tab_sym_bool marked_dart;
  int b_level,level=c_map.give_level(b);
  dart next, alpha, end=next=c_map.mu_1(v);
  do
  {
    b=next;
    b_level=c_map.give_level(b);
    if(b_level>=level || (b_level%3)!=1)
    {
      // If we are not on the kernel tree, use the sigma permutation to retrun on
      //   the kernel tree
      next=c_map.give_sigma_base(next);
      nb_cell += get_n_write_cell(b,out,c_map,dx,dy,dimy,min,marked_dart);
      continue;
    }

    // If we are on the kernel tree, use the phi permutation to move on
    //   the kernel tree
    alpha=compute_alpha_contract(b,b_level,c_map);
    next=c_map.give_sigma_base(alpha);
    nb_cell += get_n_write_cell(b,out,c_map,dx,dy,dimy,min,marked_dart);

  }
  while(next!=end);

  out.seekp(pos);
  out << nb_cell;
  out.close();
}

bool give_1_cell_coordinate(dart b,combinatorial_map &c_map,int dy,point &min,point &p)
{
  move_type m=c_map.give_move(b);
  c_map.give_point(b,p);
  if (m==right_move)
  {
    p.x = ((p.x-min.x)<<1)+1;
    p.y = (dy-p.y+min.y)<<1;
    return false;
  }
  if (m==up_move)
  {
  p.x = (p.x-min.x)<<1;
  p.y = ((dy-p.y+min.y)<<1)+1;
    return true;
  }
  if (m==left_move)
  {
  p.x = ((p.x-min.x)<<1)-1;
  p.y = (dy-p.y+min.y)<<1;
    return false;
  }
  p.x = ((p.x-min.x)<<1);
  p.y = ((dy-p.y+min.y)<<1)-1;
  return true;
}

void give_2_cell_coordinate(dart b,combinatorial_map &c_map,int dy,point &min,point &p)
{
  c_map.give_pixel(b,p);
  p.x = ((p.x-min.x)<<1)+1;
  p.y = ((dy-p.y+min.y-1)<<1)+1;
}

dart compute_alpha_contract(dart b,int l,combinatorial_map &c_map)
{
  dart sig=c_map.give_alpha_base(b);
  int level=c_map.give_level(sig);
  if ((level%3)==1 && level==l)
    return sig;

  sig=b;
  do
  {
    sig=c_map.give_sigma_base(sig);
    level=c_map.give_level(sig);
    while((level%3)!=0)
    {
      sig=c_map.give_sigma_base(c_map.give_alpha_base(sig));
      level=c_map.give_level(sig);
    }
  }
  while (c_map.give_level(c_map.give_alpha_base(sig))<l);
  
  return c_map.give_alpha_base(sig);
}

int give_lachaud_cell(int x, int y, int dx, int dy)
{
  int cell=0;

  if ( (x&1)==1 )
    cell |= 1;
  if ( (y&1)==1 )
    cell |= 2;
  cell <<= dy;
  cell |= (y>>1);
  cell <<= dx;
  cell |= (x>>1);

  return cell;
}
int get_n_write_cell(dart b,ofstream &out,combinatorial_map &c_map,int dx,int dy,int dimy,point &min,tab_sym_bool &marked_dart)
{
  int cell, mod, nb=0;
  point p;
  dart sig = c_map.give_sigma_base(b), b_tmp;
  while(sig!=b && marked_dart(sig))
    sig = c_map.give_sigma_base(sig);
  if (sig==b)
  {
    give_2_cell_coordinate(b,c_map,dimy,min,p);
    cell = give_lachaud_cell(p.x,p.y,dx,dy);
    out.write((char*)&cell,sizeof(int));
    nb++;
  }
  marked_dart.light(b);
  if (marked_dart(c_map.give_alpha_base(b)))
    return nb;

  c_map.give_point(b,p);
  give_1_cell_coordinate(b,c_map,dimy,min,p);
  if (give_1_cell_coordinate(b,c_map,dimy,min,p)==false)
  {
    b_tmp=b<0?-b:b;
    mod = b_tmp%c_map.give_width();
    if (mod==1 && !marked_dart(b_tmp-1) || !marked_dart(c_map.give_alpha_base(b_tmp-1)))
    {
      cell = give_lachaud_cell(p.x-1,p.y,dx,dy);
      out.write((char*)&cell,sizeof(int));
      nb++;
    }
    if (mod==0 || !marked_dart(b_tmp+1) || !marked_dart(c_map.give_alpha_base(b_tmp+1)))
    {
      cell = give_lachaud_cell(p.x+1,p.y,dx,dy);
      out.write((char*)&cell,sizeof(int));
      nb++;
    }
  }
  cell = give_lachaud_cell(p.x,p.y,dx,dy);
  out .write((char*)&cell,sizeof(int));
  return ++nb;
}

void lachaud_to_image(char *lachaud_file, char *image_file)
{
  ifstream in(lachaud_file);
  int dimx, dimy, dim, *d, cell, *c, *m, topo, nb_cell, i=0;

  // we read the header of the file to set the parameter
  in >> dimx;
  in >> dimy;
  in >> nb_cell;

  image res(1,dimx,dimy);
  image::iterator it=res.begin();
  in >> dim;
  d = new int [dim];
  m = new int [dim];
  c = new int [dim];
  for (int i=0; i<dim; i++)
  {
    in >> d[i];
    m[i]=0;
    for (int j=0; j<d[i]; j++)
      m[i]=(m[i]<<1)+1;
  }
  in.get();

  // we read the cell and put them on a picture
  while(in.read((char*)&cell, sizeof(int)))
  {
    i++;
    for(int i=0;i<dim;i++)
    {
      c[i] = (cell&m[i])<<1;
      cell=cell>>(d[i]);
    }
    topo = 0;
    for(int i=0;i<dim;i++)
    {
      if ((cell&1) == 1)
      {
        topo++;
        c[i]++;
      }
      cell=cell>>1;
    }
    it.move_to(c[1],c[0]);
    (*it) = topo+1;
  }

  if(i!=nb_cell)
  {
    cerr << "<< ERROR >> error on the lachaud file: " << lachaud_file << endl;
  }

  res.save(image_file);
  delete [] d;
  delete [] m;
  delete [] c;
  in.close();
}
