#include "kernel_mont_jolion.hh"

void kernel_mont_jolion::compute_kernel(combi_map_priv &c_map)
{
  // Initialization of the father array and marked dart array
  nb_contracted=0;
  contracted.reinit();
  list_label::iterator iter=c_map.vertex_set.first();
  while(iter<c_map.vertex_set.end())
  {
    Father[*iter]=*iter;
    iter++;
  }

  tab_sym_bool uncontractable;
  tab_bool survivor;
  compute_survivor(c_map, uncontractable, survivor);
  compute_merge(c_map, uncontractable, survivor);
}
bool kernel_mont_jolion::continue_computation()
{
  return nb_contracted!=0;
}

void kernel_mont_jolion::compute_survivor(combi_map_priv &c_map, tab_sym_bool &uncontractable, tab_bool &survivor)
{
  int nb_survivor=0, nb_non_survivor=0, nb_vertex=c_map.vertex_set.give_cardinal()-1;
  dart b, sig, sig_mb;
  vertex v, v_mb;
  bool min;
  double min_val;
  tab_sym_bool dart_marked;
  tab_bool vertex_marked;

  // Computation of the kernel
  do
  {
    list_label::iterator iter=c_map.vertex_set.first();
    while(iter<c_map.vertex_set.end())
    {
      v=*(iter++);
      // Seek for a local minimum
      if ((unsigned)v==c_map.give_background())
        continue;
      if (vertex_marked(v))
        continue;
      sig=b=c_map.m_1(v);
      min_val=equiv->value(v);
      min = true;
      do
      {
        sig = c_map.sigma(sig);
        if (uncontractable(sig))
	  continue;
        sig_mb = c_map.alpha(sig);
        v_mb = c_map.m(sig_mb);
	if (vertex_marked(v_mb))
	  continue;
        if ((unsigned)v_mb==c_map.give_background())
	{
          uncontractable.light(sig);
          continue;
	}
    	if(v == v_mb)
    	  continue;
        if ( !dart_marked(sig) && !equiv->lambda(level,v,v_mb,sig,sig_mb) )
	{
          dart_marked.light(sig);
          dart_marked.light(sig_mb);
          uncontractable.light(sig);
          uncontractable.light(sig_mb);
          continue;
	}
	dart_marked.light(sig);
	dart_marked.light(sig_mb);
        if( equiv->value(v_mb) < min_val )
    	{
    	  min=false;
    	  break;
    	}
      }
      while( sig!=b );


      // If this is not a local minimum
      if(!min)
        continue;

      // Then a local minimum was found
      nb_survivor++;
      sig=b=c_map.m_1(v);
      vertex_marked.light(v);
      survivor.light(v);
      do
      {
        sig = c_map.sigma(sig);
        if (uncontractable(sig))
	  continue;
	sig_mb = c_map.alpha(sig);
        v_mb = c_map.m(sig_mb);
        if ((unsigned)v_mb==c_map.give_background())
          continue;
    	if(v == v_mb)
    	  continue;
	if (vertex_marked(v_mb))
	  continue;
	vertex_marked.light(v_mb);
        nb_non_survivor++;
      }
      while( sig!=b );
    }
  }
  while((nb_survivor+nb_non_survivor)!=nb_vertex);
}

void kernel_mont_jolion::compute_merge(combi_map_priv &c_map, 
				       tab_sym_bool &uncontractable, 
				       tab_bool &survivor)
{
  dart b, mb, sig, sig_mb, b_cont;
  vertex v, v_mb;
  double min_int, curr_int;
  
  list_label::iterator iter=c_map.vertex_set.first();
  while(iter<c_map.vertex_set.end())
  {
    v=*(iter++);
    if (survivor(v))
      continue;
    if (v==(long)c_map.give_background())
      continue;
    sig=b=c_map.m_1(v);
    min_int=DBL_MAX;
    b_cont=0;
    do
    {
      sig = c_map.sigma(sig);
      if (uncontractable(sig))
        continue;
      mb = c_map.alpha(sig);
      v_mb = c_map.m(mb);
      if (!survivor(v_mb))
	continue;
      if (!equiv->lambda(level,v,v_mb,sig,mb))
        continue;

      curr_int=equiv->interest(level,v,v_mb,sig,c_map.alpha(sig));
      if(curr_int <= min_int)
      {
	b_cont = sig;
	min_int=curr_int;
      }
    }
    while( sig!=b );

    assert(b_cont!= 0);
    sig_mb = c_map.alpha(b_cont);
    contracted.light(b_cont);
    v_mb = c_map.m(sig_mb);
    Father[v]=v_mb;
    nb_contracted++;
  }
}
