/** \file kernel_mont_jolion.hh
    \brief File to compute and hold the survivor kernel of the combinatorial map */

#ifndef __KERNEL_MONT_JOLION_HH__
#define __KERNEL_MONT_JOLION_HH__

#include <float.h>
#include "kernel.hh"
#include "equivalent_mont_jolion.hh"

/** Model class to compute and hold the survivor kernel of the combinatorial map*/
class kernel_mont_jolion: public kernel
{
private:
  // Pointor on an equivalent_mont_jolion class
  equivalent_mont_jolion *equiv;

public:
  /// Constructor
  kernel_mont_jolion(equivalent_mont_jolion &equi): equiv(&equi)
  {}
  /// Destructor
  virtual ~kernel_mont_jolion()
  {}
  /// Function to compute the kernel of the combinatorial map
  void compute_kernel(combi_map_priv &);
  /// Function to know if the compute_kernel function must be iterate an other time
  bool continue_computation();
  
private:
  // Compute the survivor and contracted vertices of the map
  void compute_survivor(combi_map_priv &,tab_sym_bool &,tab_bool &);
  // Compute the most appropriate father of each non survivor vertex of the map
  void compute_merge(combi_map_priv &,tab_sym_bool &,tab_bool &);
};

#endif // __KERNEL_MONT_JOLION_HH__
