/** \file kernel_union_find.hh
    \brief File to compute and hold the survivor kernel of the combinatorial map */

#ifndef __KERNEL_UNION_FIND_HH__
#define __KERNEL_UNION_FIND_HH__

#include "kernel.hh"
#include "equivalent_union_find.hh"

/** Model class to compute and hold the survivor kernel of the combinatorial map*/
class kernel_union_find: public kernel
{
private:
  // Pointor on an equivalent_union_find class
  equivalent_union_find *equiv;

public:
  /// Constructor
  kernel_union_find(equivalent_union_find &equi): equiv(&equi)
  {}
  /// Destructor
  virtual ~kernel_union_find()
  {}
  /// Function to compute the kernel of the combinatorial map
  /// used in a do-compute_kernel while(continue_computation())
  void compute_kernel(combi_map_priv &);
  /// Function to know if the compute_kernel function must be iterate an other time
  /// used in a do-compute_kernel while(continue_computation())
  bool continue_computation();
};

#endif // __KERNEL_UNION_FIND_HH__
