#include "rm_watersheds.hh"
#include <assert.h>
#include <min_max.hh>
#include "pyramid.hh"
rm_watersheds::rm_watersheds(watershed& water): bassins(&water),nb_iter(1)
{
}
rm_watersheds::~rm_watersheds()
{
}
void rm_watersheds::contract_edge(combi_map_priv &map,
				  int max,
				  vertex v,
				  dart d)
{
  contracted.light(d);
  nb_contracted++;;
  bassins->data->set_vertex_depth(v,max);
}
void rm_watersheds::compute_kernel(combi_map_priv & map)
{
  vertex v,vp,background(map.give_background());
  vertex_iterator end_vertex_set(map.vertex_set.end());
  dart b,b0,mb;
  vertex fictitious(background);
  fifo_label levels[256];
  fifo_label* watersheds(&bassins->watershed_set);
  water_value* data(bassins->data);
  int nb_watersheds(watersheds->size());
  tab_bool in_fifo;
  int h,max(0);

 // no dart is initially contracted 
  contracted.reinit();
  nb_contracted=0;
  // each vertex is its own father
  set_father(map);
  // sort the watersheds according to their depth
  watersheds->add(fictitious);
  do
    {
      v=watersheds->first();

      if (v== fictitious)
	break;
      
      h=data->depth(v);
      // we add v to levels if it is adjacent to a bassin
      b0=b=map.m_1(v);
      do
	{	      
	  mb=map.alpha(b);
	  vp=map.m(mb);
	  if(vp==background)
	    {
	      b=map.sigma(b);
	      continue;
	    }
	  if(bassins->im0(vp)>0)
		{
		  in_fifo.light(v);
		  levels[h].add(v);
		  break;
		}
	  b=map.sigma(b);
	}
      while(b != b0);
    }
  while(1);

  for(int i=0;i<nb_watersheds;i++)
    {
      v=min_altitude(levels,255);

      h=data->depth(v);
      max=MAX(max,h);

      in_fifo.unlight(v);
      b0=b=map.m_1(v);
      do
	{	      
	  mb=map.alpha(b);
	  vp=map.m(mb);
	  if(vp==background || vp==v)
	    {
	      b=map.sigma(b);
	      continue;
	    }
	  if(bassins->im0(vp)==WSHED && (!in_fifo(vp)) )
	    {
	      in_fifo.light(vp);
	      h=data->depth(vp);
	      levels[h].add(vp);
	    }
	  else
	    {
	      if( (bassins->im0(vp) >0) && 
		  (bassins->im0(v)==WSHED)
		  )
		{
		  contract_edge(map,max,v,b);
		  Father[v]=find_compress(vp);
		  bassins->im0[v]=bassins->im0(vp);
		}
	    }
	  b=map.sigma(b);
	}
      while(b != b0);
    }

}
vertex rm_watersheds::min_altitude(fifo_label* levels,int max) throw (missed_watershed)
{
  int i(0);

  while(levels[i].empty() && (i<=max)) i++;

  if(i>max)
    throw missed_watershed("Error : the array of watershed is empty before all watershed pixels have been traversed",__FILE__,__LINE__);

  return levels[i].first();
}
bool rm_watersheds::continue_computation()
{
  return false;
}
