/** \file value_set.hh
    \brief File to manage the value data of the pyramid */

#ifndef __VALUE_SET_HH__
#define __VALUE_SET_HH__


#include "border_data.hh"
#include "vertex_data.hh"

typedef enum {cmp_watershed,rm_watershed,cmp_waterfall,cmp_dynamic} reduction_mode;

class combi_map_base;

/** Model class to manage the value data of the pyramid */
class value_set
{
protected:
  // Pointor on a combinatorial map
  combi_map_priv *map;

public:
  // Constructor
  value_set():map(0)
  {}
  /// Function that will be use to update the pointor on the combinatorial_map
  virtual void set_combinatorial_map(combi_map_priv &c_map)
  {
    map = &c_map;
  }
  /// A pure virtual function to initialize the data_set during the construction of the pyramid
  virtual void init(combi_map_base &)=0;
  /// A virtual function to manage the data when an edge will be contracted
  virtual void contract_edge(int level, combi_map_priv& map,dart d1, dart d2, vertex v1, vertex v2)
  {}
  /// A virtual function to manage the data when an empty self-loop will be remove
  virtual void remove_empty_self_loops(dart d1, dart d2, vertex v)
  {}
  /// A virtual function to manage the data when a double edge will be remove
  virtual void remove_double_edges(int level, dart d1, dart d2,dart md1)
  {}
  /// A virtual function to manage the data when vertex set is updated
  virtual void update_vertex_set(int, vertex, vertex)
  {}
  virtual void update_vertex_owner(dart d,vertex old,vertex father)
  {}

};


#endif // __VALUE_SET_HH__
