#include <assert.h>
#include "water_border_data.hh"
// The value of each edge is the maximum of the values of the two
// incident vertice.
#include <min_max.hh>
#include "combinatorial_map.hh"
#include "pyramid.hh"
#include "max_min1d.hh"

void debug_print_profil(tab_dart &,int);

water_borders::dart_data::dart_data(int value):pass_value(value),dynamic(0),cardinal(1),min(value),max(value)
{
}
void water_borders::dart_data::add(unsigned char val,int freq)
{
  cardinal+=freq;
  min=MIN(min,val);
  max=MAX(max,val);
}
void water_borders::dart_data::merge(dart_data* data)
{
  min=MIN(min,data->min);
  max=MAX(max,data->max);    
  cardinal+=data->cardinal;
}
water_borders::dart_data::~dart_data()
{
}

void water_borders::set_combinatorial_map(combi_map_priv *c_map)
{
  map=c_map;
}
void water_borders::unallocate(dart d)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data != NULL)
    delete data;
  data_border[d]=NULL;
}
int water_borders::value_lignel(dart b)
{
  vertex background(map->give_background());
  vertex v,vp;
  int depth,depth_p;

  v=map->m(b);
  vp=map->m(map->alpha(b));
  
  
  if (v==background)
    return v_data->give_depth(vp);
  
  if(vp==background)
    return v_data->give_depth(v);
      

  depth=v_data->give_depth(v);
  depth_p=v_data->give_depth(vp);
  return MAX(depth,depth_p);
}

void water_borders::contract_edge(combi_map_priv& map,vertex v,dart d)
{
}
void water_borders::merge_dynamics(int level,dart b,dart bp,dart mb)
{
  dart_data * data_b,*data_mb,*data_bp;
  dart alpha_b,alpha_mb,alpha_bp;



  // compute data_b
  alpha_b=map->alpha(b);
  data_b=(dart_data*)data_border(b);
  if(!data_b)
    data_b=(dart_data*)data_border(alpha_b);

  if(!data_b)
    return;

  // compute data_mb
  alpha_mb=map->alpha(mb);  
  data_mb=(dart_data*)data_border(mb);
  if(!data_mb)
    data_mb=(dart_data*)data_border(alpha_mb);


  alpha_bp=map->alpha(bp);

  if(data_mb)
    {
      data_b->merge(data_mb);
    }
  if(bp != alpha_mb)
    {	        
      data_bp=(dart_data*)data_border(bp);  
      if(!data_bp)
	data_bp=(dart_data*)data_border(alpha_bp);  
      data_b->merge(data_bp);
    }
}  

void water_borders::update_pass_values(int level,dart b,dart bp,dart mb)
{
 
  dart_data * data_b,*data_mb,*data_bp;
  dart alpha_b,alpha_mb,alpha_bp;

  alpha_b=map->alpha(b);

  data_b=(dart_data*)data_border(b);
  if(!data_b)
    data_b=(dart_data*)data_border(alpha_b);

  if(!data_b)
    return;

  alpha_mb=map->alpha(mb);  
  data_mb=(dart_data*)data_border(mb);
  if(!data_mb)
    data_mb=(dart_data*)data_border(alpha_mb);


  alpha_bp=map->alpha(bp);

  if(data_mb)
    {
      data_b->pass_value=MIN(data_b->pass_value,data_mb->pass_value);
    }
  if(bp != alpha_mb)
    {	        
      data_bp=(dart_data*)data_border(bp);  
      if(!data_bp)
	data_bp=(dart_data*)data_border(alpha_bp);  
      data_b->pass_value=MIN(data_b->pass_value,data_bp->pass_value);   
    }
}
void water_borders::compute_pass_values(int level,dart b,dart bp,dart mb)
{
  dart_data * data_b,*data_mb,*data_bp;
  int value_b,value_bp;
  data_b=(dart_data*)data_border(b);
  data_mb=(dart_data*)data_border(mb);
  data_bp=(dart_data*)data_border(bp);
  
  // both null
  if(data_b ==NULL && data_mb ==NULL)
    {
      value_b= value_lignel(b);
      value_bp= value_lignel(bp);
      data_b=new dart_data(value_b);

      assert((value_b>=0 &&value_b<256));
      assert((value_bp>=0 &&value_bp<256));
      data_b->add(value_bp);
      data_border[b]=data_b;
      return;
    }

  if(data_mb != NULL)
    return;


  value_bp= value_lignel(bp);
  assert((value_bp>=0 &&value_bp<256));
  data_b->add(value_bp);
}
void water_borders::merge(int level,dart b,dart bp,dart mb)
{

  switch(mode)
    {
    case cmp_watershed:
      break;
    case rm_watershed:
      compute_pass_values(level,b,bp,mb);
      break;
   case cmp_waterfall:
      update_pass_values(level,b,bp,mb);
      break;
    case cmp_dynamic:
      merge_dynamics(level,b,bp,mb);
      break;
    }
  if(mode==cmp_waterfall || mode==cmp_dynamic)
    {
        dart_data * data_b,*data_mb,*data_bp;
	dart alpha_b,alpha_mb,alpha_bp;


	alpha_b=map->alpha(b);
	data_b=(dart_data*)data_border(b);
	if(!data_b)
	  data_b=(dart_data*)data_border(alpha_b);

	if(!data_b)
	  return;
	alpha_mb=map->alpha(mb);  

	data_mb=(dart_data*)data_border(mb);
	if(!data_mb)
	  data_mb=(dart_data*)data_border(alpha_mb);

	if(data_mb)
	  {

	    delete data_mb;
	    data_border[mb]=data_border[alpha_mb]=NULL;
	    if(data_border(b)==NULL)
	      {
		data_border[b]=data_b;
		data_border[alpha_b]=NULL;
	      }
	  }
	if(bp != alpha_mb)
	  {	        
	    alpha_bp=map->alpha(bp);
	    data_bp=(dart_data*)data_border(bp);  
	    if(!data_bp)
	      data_bp=(dart_data*)data_border(alpha_bp);  
	    delete data_bp;
	    data_border[bp]=data_border[alpha_bp]=NULL;        
	  }  
    }
}
void water_borders::update_vertex_owner(dart d,
					vertex v,
					vertex father)
{
  dart_data *data_d,*data_md;

  if(mode!=rm_watershed)
    return;
  
  data_d=(dart_data*)data_border(d);
  data_md=(dart_data*)data_border(map->alpha(d));

  if(data_d == NULL && data_md == NULL)
    data_border[d]=new dart_data(value_lignel(d));
}
void water_borders::set_mean(dart d)
{

  dart_data *data;
  vertex m_b,m_mb,background(map->give_background());
  dart md;

    
  data=(dart_data*)data_border(d);

  if(data == NULL)
    return;
  
  if(data->cardinal==1)
    return;

  m_b=map->mu(d);
  md=map->alpha(d);
  m_mb=map->mu(md);
  
  if(m_b==background|| m_mb==background)
    {
      data->pass_value=255;
      return;
    }
  // test pour l'article. 
  //  data->pass_value=data->min;
  
  if(data->cardinal > 5)
    data->pass_value=min_with_max_volume(d);
  else
    data->pass_value=data->min;//median(d,data);
  
} 

int water_borders::lignel_grad(segment_iterator &ite,
			       image::iterator & it_grad)
{
  point p1,p2;
  int val1,val2;

  ite.give_pixel(p1,p2);

  it_grad.move_to(p1.y,p1.x);
  val1=*it_grad;

  it_grad.move_to(p2.y,p2.x);
  val2=*it_grad;

  return MAX(val1,val2);
}
int water_borders::median(dart d,dart_data* data)
{
  int histo[256];
  segment seg(d,map->alpha(d),
	      map->pyr->give_max_level(),
	      map->pyr);
  segment_iterator end=seg.end();  
  image::iterator it_grad(*img_gradient);
  
  // initialisation of the histogram
  memset((void*)histo,0,256*sizeof(int));
  
  for(segment_iterator ite(&seg);ite!=end;ite++)
      histo[lignel_grad(ite,it_grad)]++;

  // computation of the median
  int median(data->cardinal/2);
  int i(0),card(0);
  while((card < median)&&(i<256))
    {
      card+=histo[i];
      i++;
    }

  return i;
}
  
int water_borders::min_with_max_volume(dart d)
{
  static tab_dart profil;
  static tab_ptr tab_extr,min_set;
  int hist_max[256];
  int previous,count;
  mini* min_min(NULL);    
  segment seg(d,map->alpha(d),
	      map->pyr->give_max_level(),
	      map->pyr);
  segment_iterator last=seg.last();  
  image::iterator it_grad(*img_gradient);

  memset(hist_max,0,256*sizeof(int));

  int nb(0);
  int max_profil(-1);
  int min_debug(255);
  for(segment_iterator ite(&seg);ite!=last;ite++)
    {
      profil[nb]=lignel_grad(ite,it_grad);
      if(max_profil<profil(nb))
	max_profil=profil(nb);
      if(min_debug>profil(nb))
	min_debug=profil(nb);
      nb++;
    }
  max_profil++;
  profil[nb]=lignel_grad(last,it_grad);
  nb++;

  mini *cur_min;
  maxi* cur_max;
  int i=0;
  tab_extr[0]=(any_ptr)alloc_maxi(max_profil,-1);
  hist_max[max_profil]=1;
  count=1;

  i=0;
  while(i<nb)
    {
      do
	{
	  previous=profil[i];
	  i++;
	}
      while(i<nb&&profil[i]<=previous);

      cur_min=alloc_mini(profil[i-1],i-1);

      cur_min->left=(max_min*)tab_extr[count-1];
      ((max_min*)tab_extr[count-1])->right=cur_min;
      tab_extr[count]=(any_ptr)cur_min;
      count++;
      
      cur_min->end=i-1;
      int j(i-2);
      while(j>0&&profil[j]==cur_min->value)j--;
      cur_min->begin=j+1;

      if(i==nb)
	{
	  cur_min->end=i-1;
	  cur_max=alloc_maxi(max_profil,nb);
	  cur_max->left=cur_min;
	  cur_min->right=cur_max;
	  tab_extr[count]=(any_ptr)cur_max;
	  hist_max[max_profil]++;
	  count++;
	  break;
	}
      do
	{
	  previous=profil[i];
	  i++;
	}
      while(i<nb&&profil[i]>=previous);
      cur_max=alloc_maxi(previous,i-1);
      cur_max->left=(max_min*)tab_extr[count-1];
      ((max_min*)tab_extr[count-1])->right=cur_max;
      tab_extr[count]=(any_ptr)cur_max;
      count++;
      if(i==nb)
	{
	  cur_max->value=max_profil;
	  hist_max[max_profil]++;
	  break;
	}
      hist_max[cur_max->value]++;
    }
  // If there is only one minimum we return its value
  if(count==3)
    return ((mini*)tab_extr[1])->value;
  // Otherwize we compute the minimum with the maximal volume.
  int cursors[256];
  cursors[0]=0;
  int card(hist_max[0]);
  for(i=1;i<256;i++)
    {
      cursors[i]=card;
      card+=hist_max[i];
    }
  tab_ptr max_set;
  for(int i=0;i<=count;i+=2)
  {
    cur_max=(maxi*)tab_extr[i];
    max_set[cursors[cur_max->value]]=(any_ptr)cur_max;
    cursors[cur_max->value]++;
  }
  i=0;
  while(((maxi*)max_set[i])->value != max_profil)
    {
    cur_max=(maxi*)max_set[i];
    remove_max(cur_max,min_min,profil,nb);    
    i++;
  }
  
  int minimal_value(min_min->value);
  maxi* max=(maxi*)tab_extr[0];
  mini* min;
  while(max->right != NULL)
    {
      min=(mini*)max->right;
      min->compute_volume(profil,nb);
      if(min->volume >  min_min->volume)
	{
	  //	  minimal_value=min->value;
	}
      delete_maxi(max);
      max=(maxi*)min->right;
      delete_mini(min);
    }
  delete_mini(min_min);
  return minimal_value;
}
int water_borders::pass_value(dart d)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data == NULL)
    return -1;
  return data->pass_value;
}
void water_borders::set_dynamic(dart d,int dyn) throw (null_data_field)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data == NULL)
    throw null_data_field("Acess to a null data field",__FILE__,__LINE__);
  data->dynamic=dyn;
  //    data->dynamic=data->pass_value;
  //  cerr <<d<<" : "<<data->pass_value<<"->"<<dyn<<endl;
}
int water_borders::dynamic(dart d)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data == NULL)
    return -1;
  return data->dynamic;
}
int water_borders::min(dart d)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data == NULL)
    return -1;
  return data->min;
}
void debug_print_profil(tab_dart &profil,int nb)
{
  for(int i=0;i<nb;i++)
    cerr << i<<" "<<profil[i]<<endl;
}
