#include "water_border_data.hh"
// The value of each edge is the maximum of the values of the two
// incident vertice.
#include <min_max.hh>

water_borders::dart_data::dart_data(int value):pass_value(value),dynamic(0),cardinal(1),histo(NULL),min(value),max(value)
{
}
void water_borders::dart_data::add(unsigned char val,int freq)
{
  if(cardinal==1)
    {
      histo=(int*)calloc(256,sizeof(int));
      assert(histo);
      histo[min]++;
    }      
  cardinal+=freq;
  histo[val]+=freq;
  min=MIN(min,val);
  max=MAX(max,val);
}
void water_borders::dart_data::merge(dart_data* data)
{


  if(data->cardinal==1)
    {
      add(data->min);
      return;
    }

  if(cardinal==1)
    {
      histo=data->histo;
      data->histo=NULL;
      histo[min]++;
      min=MIN(min,data->min);
      max=MAX(max,data->max);    
      cardinal=data->cardinal+1;
      data->cardinal=1;
      return;
    }
  for(int i=data->min;i<=data->max;i++)
    histo[i]+=data->histo[i];
  min=MIN(min,data->min);
  max=MAX(max,data->max);    
  cardinal+=data->cardinal;
}
water_borders::dart_data::~dart_data()
{
  if(histo)
      free(histo);
}

void water_borders::set_combinatorial_map(combi_map_priv *c_map)
{
  map=c_map;
}
void water_borders::unallocate(dart d)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data != NULL)
    delete data;
  data_border[d]=NULL;
}
int water_borders::value_lignel(dart b)
{
  vertex background(map->give_background());
  vertex v,vp;
  int depth,depth_p;

  v=map->m(b);
  vp=map->m(map->alpha(b));
  
  
  if (v==background)
    return v_data->give_depth(vp);
  
  if(vp==background)
    return v_data->give_depth(v);
      

  depth=v_data->give_depth(v);
  depth_p=v_data->give_depth(vp);
  return MAX(depth,depth_p);
}

void water_borders::contract_edge(combi_map_priv& map,vertex v,dart d)
{
}
void water_borders::merge_dynamics(int level,dart b,dart bp,dart mb)
{
  dart_data * data_b,*data_mb,*data_bp;
  dart alpha_b,alpha_mb,alpha_bp;



  // compute data_b
  alpha_b=map->alpha(b);
  data_b=(dart_data*)data_border(b);
  if(!data_b)
    data_b=(dart_data*)data_border(alpha_b);

  if(!data_b)
    return;

  // compute data_mb
  alpha_mb=map->alpha(mb);  
  data_mb=(dart_data*)data_border(mb);
  if(!data_mb)
    data_mb=(dart_data*)data_border(alpha_mb);


  alpha_bp=map->alpha(bp);

  if(data_mb)
    {
      data_b->merge(data_mb);
    }
  if(bp != alpha_mb)
    {	        
      data_bp=(dart_data*)data_border(bp);  
      if(!data_bp)
	data_bp=(dart_data*)data_border(alpha_bp);  
      data_b->merge(data_bp);
    }
}  

void water_borders::update_pass_values(int level,dart b,dart bp,dart mb)
{
 
  dart_data * data_b,*data_mb,*data_bp;
  dart alpha_b,alpha_mb,alpha_bp;

  alpha_b=map->alpha(b);

  data_b=(dart_data*)data_border(b);
  if(!data_b)
    data_b=(dart_data*)data_border(alpha_b);

  if(!data_b)
    return;

  alpha_mb=map->alpha(mb);  
  data_mb=(dart_data*)data_border(mb);
  if(!data_mb)
    data_mb=(dart_data*)data_border(alpha_mb);


  alpha_bp=map->alpha(bp);

  if(data_mb)
    {
      data_b->pass_value=MIN(data_b->pass_value,data_mb->pass_value);
    }
  if(bp != alpha_mb)
    {	        
      data_bp=(dart_data*)data_border(bp);  
      if(!data_bp)
	data_bp=(dart_data*)data_border(alpha_bp);  
      data_b->pass_value=MIN(data_b->pass_value,data_bp->pass_value);   
    }
}
void water_borders::compute_pass_values(int level,dart b,dart bp,dart mb)
{
  dart_data * data_b,*data_mb,*data_bp;
  int value_b,value_bp;
  data_b=(dart_data*)data_border(b);
  data_mb=(dart_data*)data_border(mb);
  data_bp=(dart_data*)data_border(bp);
  
  // both null
  if(data_b ==NULL && data_mb ==NULL)
    {
      value_b= value_lignel(b);
      value_bp= value_lignel(bp);
      data_b=new dart_data(value_b);

      assert((value_b>=0 &&value_b<256));
      assert((value_bp>=0 &&value_bp<256));
      data_b->add(value_bp);
      data_border[b]=data_b;
      return;
    }

  if(data_mb != NULL)
    return;

  assert(data_b->histo);
  value_bp= value_lignel(bp);
  assert((value_bp>=0 &&value_bp<256));
  data_b->add(value_bp);
}
void water_borders::merge(int level,dart b,dart bp,dart mb)
{

  switch(mode)
    {
    case cmp_watershed:
      break;
    case rm_watershed:
      compute_pass_values(level,b,bp,mb);
      break;
   case cmp_waterfall:
      update_pass_values(level,b,bp,mb);
      break;
    case cmp_dynamic:
      merge_dynamics(level,b,bp,mb);
      break;
    }
  if(mode==cmp_waterfall || mode==cmp_dynamic)
    {
        dart_data * data_b,*data_mb,*data_bp;
	dart alpha_b,alpha_mb,alpha_bp;


	alpha_b=map->alpha(b);
	data_b=(dart_data*)data_border(b);
	if(!data_b)
	  data_b=(dart_data*)data_border(alpha_b);

	if(!data_b)
	  return;
	alpha_mb=map->alpha(mb);  

	data_mb=(dart_data*)data_border(mb);
	if(!data_mb)
	  data_mb=(dart_data*)data_border(alpha_mb);

	if(data_mb)
	  {

	    delete data_mb;
	    data_border[mb]=data_border[alpha_mb]=NULL;
	    if(data_border(b)==NULL)
	      {
		data_border[b]=data_b;
		data_border[alpha_b]=NULL;
	      }
	  }
	if(bp != alpha_mb)
	  {	        
	    alpha_bp=map->alpha(bp);
	    data_bp=(dart_data*)data_border(bp);  
	    if(!data_bp)
	      data_bp=(dart_data*)data_border(alpha_bp);  
	    delete data_bp;
	    data_border[bp]=data_border[alpha_bp]=NULL;        
	  }  
    }
}
void water_borders::update_vertex_owner(dart d,
					vertex v,
					vertex father)
{
  dart_data *data_d,*data_md;

  if(mode!=rm_watershed)
    return;
  
  data_d=(dart_data*)data_border(d);
  data_md=(dart_data*)data_border(map->alpha(d));

  if(data_d == NULL && data_md == NULL)
    data_border[d]=new dart_data(value_lignel(d));
}
void water_borders::set_mean(dart d)
{

  int i(0);
  dart_data *data;
  int mean;

  data=(dart_data*)data_border(d);

  if(data == NULL)
    return;
  
  if(data->cardinal==1)
    return;

  //  data->pass_value=min;
  //  return;
  i=data->min;
  int midle(data->cardinal/2);
  int card(0);
  while( (i<data->max)&&(card<midle) )
    {
      card+=data->histo[i];
      i++;
    }
  mean=i;
  if(data->max-data->min >15)
    data->pass_value=data->min;
  else
    data->pass_value=mean;
}
  
int water_borders::pass_value(dart d)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data == NULL)
    return -1;
  return data->pass_value;
}
void water_borders::set_dynamic(dart d,int dyn) throw (null_data_field)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data == NULL)
    throw null_data_field("Acess to a null data field",__FILE__,__LINE__);
  data->dynamic=dyn;
  //    data->dynamic=data->pass_value;
  //  cerr <<d<<" : "<<data->pass_value<<"->"<<dyn<<endl;
}
int water_borders::dynamic(dart d)
{
  dart_data *data=(dart_data*)data_border(d);

  if(data == NULL)
    return -1;
  return data->dynamic;
}
