#include "water_value.hh"

#define MIN(a,b) ((a)<(b)?(a):(b))
#define MAX(a,b) ((a)>(b)?(a):(b))

void water_value::init(combi_map_base & map)
{ 
  vertex bg=map.give_background();
  for (vertex v=bg-1;v>0;v--)
    v_data->init(map,v);

}
void water_value::update_vertex_set(int level, 
				    vertex v1, 
				    vertex v2)
{
  if (v1==v2)
    return;

  v_data->merge(level,v1,v2);
}
void water_value::set_vertex_depth(vertex v, int depth)
{
  
  v_data->set_depth(v,depth);
}
void water_value::remove_double_edges(int level, dart d, dart dp,dart md)
{
  b_data.merge(level,d,dp,md);
}
void water_value::remove_empty_self_loops(dart d, dart md, vertex v)
{
  b_data.unallocate(d);
  b_data.unallocate(md);
}
void water_value::set_labels(tab_label& label,int flag)
{
  watershed=&label;
  wshed_flag=flag;
}
void water_value::set_mean(pyramid &P)
{
  int max_level=P.give_max_level();
  dart b;

  for(dart_iterator ite=P.first_dart(max_level);
      ite<P.end_dart(max_level);
      ite++)
    {
      b=*ite;
      b_data.set_mean(b);
    }      
}
void water_value::compute_dynamics(int & min_dyn)
{
  int intervals[256];
  int cursors[256];
  tab_label father;
  tab_dart pass_values;
  dart b;
  int value;
  vertex v,vp,f_v,f_vp;
  int depth,min_depth,depth_p;
  int background(map->give_background());

  min_dyn=255;
  set_mode(cmp_dynamic);
  // we update pass values
  set_mean(*map->pyr);
  // each vertex is initially is own father
  for(vertex_iterator ite=map->vertex_set.first();
      ite<map->vertex_set.end();
      ite++)
    {
      v=*ite;
      if(v==background)
	continue;

      father[v]=v;
    }
  // we compute the number of edges for each pass_value
  for(int i=0;i<256;i++)
    intervals[i]=0;

  int max_level=map->pyr->give_max_level();
  for(dart_iterator ite=map->pyr->first_dart(max_level);
      ite<map->pyr->end_dart(max_level);
      ite++)
    {
      b=*ite;
      value=b_data.pass_value(b);
      if(value != -1)
	intervals[value]++;
    }
  // we set a cursor for each pass_value
  cursors[0]=0;
  int card(intervals[0]);
  for(int i=1;i<256;i++)
    {      
      cursors[i]=card;
      card+=intervals[i];
    }

  // we store the edges by sorting them according to their pass value.
  for(dart_iterator ite=map->pyr->first_dart(max_level);
      ite<map->pyr->end_dart(max_level);
      ite++)
    {
      b=*ite;
      value=b_data.pass_value(b);
      if(value != -1)
	{
	  pass_values[cursors[value]]=b;
	  cursors[value]++;
	}  
    }
  // we scan the set of edges in ascending order of their pass values
  // and set at each stage the dynamic of the edge.

  for(int i=0;i<card;i++)
    {
      b=pass_values(i);
      v=map->m(b);
      vp=map->m(map->alpha(b));

      if(v==vp|| v==background || vp==background)
	continue;
      
      f_v=find_compress(father,v);
      f_vp=find_compress(father,vp);
      depth=v_data->give_depth(f_v);
      depth_p=v_data->give_depth(f_vp);
      value=b_data.pass_value(b);
      if(f_v==f_vp)
	{
	  b_data.set_dynamic(b,value-depth);
	  min_dyn=MIN(min_dyn,value-depth);
	  continue;
	}
      if(depth<depth_p)
	{
	  min_depth=depth_p;
	  father[f_vp]=f_v;
	}
      else
	{
	  min_depth=depth;
	  father[f_v]=f_vp;
	}
      b_data.set_dynamic(b,value-min_depth);
      min_dyn=MIN(min_dyn,value-min_depth);
    }
  
}
vertex water_value::find_compress(tab_label &father,vertex v)
{
  vertex vp=v;

  while(father(vp) != vp)
    vp=father(vp);
  return vp;
}


int water_value::dynamic(dart d)
{
  return b_data.dynamic(d);
}
int water_value::pass_value(dart d)
{
  return b_data.pass_value(d);
}
