#include "waterfall.hh"

waterfall::waterfall(water_value& values,int val): data(&values),threshold(val)
{
}
void waterfall::compute_kernel(combi_map_priv & map)
{
  vertex v,vp,f_v,f_vp,background(map.give_background());
  dart b,mb;
  int alt_b,alt_v;

  nb_contracted=0;
  contracted.reinit();

  set_father(map);
  
  vertex_iterator end(map.vertex_set.end());
  for (vertex_iterator v_iter=map.vertex_set.first();
       v_iter<end;
       v_iter++)
    {
      v=(*v_iter);
      if (v==background)
	continue;
      b=min_altitude(map,v,alt_b);
      alt_v=data->depth(v);
      if(alt_b-alt_v > threshold)
	continue;
      mb=map.alpha(b);
      vp=map.m(mb);
      if(vp==background)
	continue;
      f_v=find_compress(v);
      f_vp=find_compress(vp);
      if(f_v!=f_vp)
	{
	  Father[v]=Father[f_v]=f_vp;
	  contract_edge(b,mb);
	}
    }
}

bool waterfall::continue_computation()
{
  return false;
}
dart waterfall::min_altitude(combi_map_priv & map,vertex v,int& min)
{
  dart b, b0,b_min,mb;
  int alt,min_alt=256;
  int alt_vp,alt_v=data->depth(v);
  vertex vp,background(map.give_background());

  b=b0=map.m_1(v);
  do
    {
      alt=data->pass_value(b);
      mb=map.alpha(b);
      vp=map.m(mb);
      if(vp==background)
	{
	  b=map.sigma(b);
	  continue;
	}

      alt_vp=data->depth(vp);
      if (alt == -1)
	{
	  alt=data->pass_value(mb);
	}
      alt=(alt- alt_v +alt-alt_vp)/2;
      if(alt<min_alt)
	{
	  min_alt=alt;
	  b_min=b;
	}
      b=map.sigma(b);
    }
  while(b != b0);

  min=min_alt;
  return b_min;    
}
